/////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright 2007-2008 Dapfor
//                                                                         
// This file is part of the Dapfor software, and may only be used, modified, and distributed 
// under the terms of the Dapfor Software License, Version 1.0.
// (See accompanying file lisense.txt or open the license at www.dapfor.com/license_agreement.aspx)
// By continuing to use, modify, or distribute this file you indicate that you have read the license 
// and understand and accept it fully.
/////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <Dapfor/GUI/GUI_Export.h>
#include <Dapfor/GUI/IAutoDelete.h>

namespace Dapfor
{
namespace GUI
{
class CGrid;

//////////////////////////////////////////////////////////////////////////
///\brief Interface to perform some actions (copy, paste, navigation, etc.) in the grid in response 
// to key pressing.
///
/// Navigation handling, keys pressing reaction are carried off the grid in a separate module. 
/// The grid behavior can be completely redefined in IKeyManager interface. GUI library gives a default realization 
/// of given interface. The programmer can create his own realization, inheriting from CKeyManager or IKeyManager. 
/// The grid gives CGrid::SetKeyManager() function which allows to set a new manager. In the case when the manager 
/// is not set, the grid  uses the default realization, returned by CPreferences::GetDefaultKeyManager().
///
/// \class  IKeyManager IKeyManager.h Dapfor/GUI/IKeyManager.h
/// \file IKeyManager.h
//////////////////////////////////////////////////////////////////////////
class DF_GUI_API IKeyManager : virtual public IAutoDelete 
{
public:
    /// Called when a non-system key is pressed
    virtual BOOL OnKeyDown(CGrid& grid, UINT nChar, UINT nRepCnt, UINT nFlags) = 0;
    
    /// Called when a non-system key is released
    virtual BOOL OnKeyUp(CGrid& grid, UINT nChar, UINT nRepCnt, UINT nFlags) = 0;

    ///\brief Moves a focus up and scrolls down one line according to specified 
    ///  values of bShift and bCtrl and navigation settings
    virtual void NavigateKeyUp(CGrid& grid, bool bShift, bool bCtrl) = 0;

    ///\brief Moves a focus down and scrolls up one line according to specified 
    ///  values of bShift and bCtrl and navigation settings
    virtual void NavigateKeyDown(CGrid& grid, bool bShift, bool bCtrl) = 0;

    ///\brief Moves a focus up and scrolls down one page according to specified 
    ///  values of bShift and bCtrl and navigation settings
    virtual void NavigateKeyPageUp(CGrid& grid, bool bShift, bool bCtrl) = 0;

    ///\brief Moves a focus down and scrolls up one page according to specified 
    ///  values of bShift and bCtrl and navigation settings.
    virtual void NavigateKeyPageDown(CGrid& grid, bool bShift, bool bCtrl) = 0;

    ///\brief Focuses the first line in the fixed or scrollable context according to specified 
    ///  values of bShift and bCtrl and navigation settings.
    virtual void NavigateKeyHome(CGrid& grid, bool bShift, bool bCtrl) = 0;

    ///\brief Focuses the last line in the fixed or scrollable context according to specified 
    ///  values of bShift and bCtrl and navigation settings.
    virtual void NavigateKeyEnd(CGrid& grid, bool bShift, bool bCtrl) = 0;
    
    ///\brief Collapses lines and moves a focus up according to specified 
    ///  values of bShift and bCtrl and navigation settings.
    virtual void NavigateKeyLeft(CGrid& grid, bool bShift, bool bCtrl) = 0;

    ///\brief Expands lines and moves a focus down according to specified 
    ///  values of bShift and bCtrl and navigation settings.
    virtual void NavigateKeyRight(CGrid& grid, bool bShift, bool bCtrl) = 0;
};


}//namespace GUI

}//namespace Dapfor

