/////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright 2007-2008 Dapfor
//                                                                         
// This file is part of the Dapfor software, and may only be used, modified, and distributed 
// under the terms of the Dapfor Software License, Version 1.0.
// (See accompanying file lisense.txt or open the license at www.dapfor.com/license_agreement.aspx)
// By continuing to use, modify, or distribute this file you indicate that you have read the license 
// and understand and accept it fully.
/////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <Dapfor/GUI/GUI_Export.h>
#include <Dapfor/GUI/IKeyManager.h>

namespace Dapfor
{
namespace GUI
{
class CGrid;

//////////////////////////////////////////////////////////////////////////
///\brief Default implementation of IKeyManager interface
///
/// The given manager processes the following events:
/// - Escape:         Removes a selection, takes a focus away
/// - KeyUp:          Moves a focus one line up
/// - KeyDown:        Moves a focus one line down
/// - PageUp:         Moves a focus one page up 
/// - PageDown:       Moves a focus one page down
/// - Home:           Moves a focus at the beginning of a context (fixed or scrollable)
/// - End:            Moves a focus at the end of a context (fixed or scrollable)
/// - Space:          Moves a focus one page down
/// - Shift+Space:    Moves a focus one page up
/// - KeyLeft:        Collapses children for a focused line or moves a focus one line up 
/// - KeyRight:       Expands children for a focused line or moves a focus one line down
/// - Return:         Collapses or expands children for a focused line
/// - Ctrl+A, Ctrl+a: Selects all lines in the grid
/// - Ctrl+C, Ctrl+c: Copies a selection to the clipboard
/// - Ctrl+X, Ctrl+x: Cuts a selection to the clipboard 
/// - Ctrl+V, Ctrl+v: Inserts data from the clipboard into the grid
///
/// Shift key pressing with simultaneous KeyUp KeyDown pressing, allows to invert a selection of many lines. 
/// The given manager allows to carry out navigation and selection as in one context, so simultaneously in both of them.
///
/// \class  CKeyManager KeyManager.h Dapfor/GUI/KeyManager.h
/// \file KeyManager.h
//////////////////////////////////////////////////////////////////////////
class DF_GUI_API CKeyManager : public IKeyManager
{
public:
    CKeyManager(bool autoDelete = false);   //!< Constructor
    ~CKeyManager();                         //!< Destructor


    // Called when a nonsystem key is pressed
    virtual BOOL OnKeyDown(CGrid& grid, UINT nChar, UINT nRepCnt, UINT nFlags);
    
    // Called when a nonsystem key is released
    virtual BOOL OnKeyUp(CGrid& grid, UINT nChar, UINT nRepCnt, UINT nFlags);

    // Moves a focus up and scrolls down one line according to specified 
    // values of bShift and bCtrl and navigation settings
    virtual void NavigateKeyUp(CGrid& grid, bool bShift, bool bCtrl);
    
    // Moves a focus down and scrolls up one line according to specified 
    // values of bShift and bCtrl and navigation settings
    virtual void NavigateKeyDown(CGrid& grid, bool bShift, bool bCtrl);
    
    // Moves a focus up and scrolls down one page according to specified 
    // values of bShift and bCtrl and navigation settings
    virtual void NavigateKeyPageUp(CGrid& grid, bool bShift, bool bCtrl);
    
    // Moves a focus down and scrolls up one page according to specified 
    // of bShift and bCtrl and navigation settings.
    virtual void NavigateKeyPageDown(CGrid& grid, bool bShift, bool bCtrl);
    
    // Focuses the first line in the fixed or scrollable context according to specified 
    // values of bShift and bCtrl and navigation settings.
    virtual void NavigateKeyHome(CGrid& grid, bool bShift, bool bCtrl);

    // Focuses the last line in the fixed or scrollable context according to specified 
    // values of bShift and bCtrl and navigation settings.
    virtual void NavigateKeyEnd(CGrid& grid, bool bShift, bool bCtrl);

    // Collapses lines and moves a focus up according to specified 
    // values of bShift and bCtrl and navigation settings.
    virtual void NavigateKeyLeft(CGrid& grid, bool bShift, bool bCtrl);

    // Expands lines and moves a focus down according to specified 
    // values of bShift and bCtrl and navigation settings.
    virtual void NavigateKeyRight(CGrid& grid, bool bShift, bool bCtrl);

    /// Handles 'Escape' key pressing
    virtual void OnEscape(CGrid& grid);

    /// Handles 'Return' key pressing
    virtual void OnReturn(CGrid& grid);
    
    /// Handles 'Ctri+A' key pressing. Selects all contents in the grid
    virtual void OnSelectAll(CGrid& grid);

    /// Handles 'Ctri+C' key pressing. Copies a selection to the clipboard
    virtual void OnCopySelection(CGrid& grid);

    /// Handles 'Ctri+X' key pressing. Cuts a selection to the clipboard
    virtual void OnCutSelection(CGrid& grid);

    /// Handles 'Ctri+V' key pressing. Copies a clipboard content to the grid.
    virtual void OnPasteSelection(CGrid& grid);

    // IAutoDelete implementation
    virtual bool AutoDelete() const;

protected:
    bool m_AutoDelete;

private:
    //Prevent from copy
    CKeyManager(const CKeyManager&);
    CKeyManager operator = (const CKeyManager&);
};


}//namespace GUI

}//namespace Dapfor


